package model;

/**
 * ADT insieme, presenta le propriet tipiche dell'insieme: aggrega elementi 
 * tutti dello stesso tipo, non esiste un legame posizionale fra gli elementi,
 * non  possibile avere elementi duplicati nell'insieme. 
 * Sono definite alcune operazioni fra insiemi e fra elemento e insieme.
 * Eventuali eccezioni saranno di tipo uncatched
 * Nota Bene: <T extends Comparable<T>> implica che T dispone del metodo compareTo()
 * @author giovanni.ragno
 */
public interface Insieme<T extends Comparable<T>> {
  
  /**
   * Operazione per includere nell'insieme l'elemento passato come parametro.
   * Non sono ammessi duplicati nell'insieme, in caso esista gi elem non si fa 
   * alcuna azione, senza nemmeno produrre eccezioni.
   * @param elem il dato da inserire nell'insieme
   */
  public void includi(T elem);
  
  /**
   * Operazione per rimuovere dall'insieme l'elemento passato come parametro.
   * La richiesta di escludere un elemento non presente nell'insieme non produce 
   * errore.
   * @param elem il dato da rimuovedere dall'insieme
   */
  public void escludi(T elem);
  
  /**
   * Operazione che restituisce un insieme contenente gli elementi presenti sia 
   * nell'insieme (this) sia in quello passato come parametro.
   * L'insieme (this) n quello passato non vengono emodificatI.
   * @param ins il secondo insieme su cui operare
   * @return il nuovo insieme ottenuto dall'intersezione.
   */
  public Insieme<T> intersezione(Insieme<T> ins);
  
  /**
   * Operazione che restiuisce un insieme contenente gli elementi presenti 
   * indifferentemente nell'insieme (this) oppure in quello passato come parametro.
   * L'insieme (this) n quello passato non vengono emodificatI.
   * @param ins il secondo insieme su cui operare.
   * @return il nuovo insieme ottenuto dall'unione.
   */
  public Insieme<T> unione(Insieme<T> ins);
  
  /**
   * Operazione che restituisce un insieme contenente gli elementi che sono 
   * presenti nell'insieme (this) e non sono presenti nell'insieme passato come 
   * parametro. Dagli elementi presenti nell'insieme (this) cio sono esclusi 
   * gli elementi presenti anche nel secondo insieme.
   * L'insieme (this) n quello passato non vengono emodificatI.
   * @param ins il secondo insieme su cui operare
   * @return il nuovo insieme ottenuto dalla differenza
   */
  public Insieme<T> differenza(Insieme<T> ins);
  
  /**
   * Operazione per determinare se l'elemento passato come parametro appartiene 
   * all'insieme.
   * @param elem l'elemento da testare nell'insieme.
   * @return true se elem  presente nell'insieme.
   */
  public boolean appartiene(T elem);
  
  /**
   * operazione per testare se l'insieme  vuoto
   * @return true se l'insieme non ha elementi.
   */
  public boolean isEmpty();
  
  /**
   * Restituisce un nuovo insieme che ha gli stessi elementi dell'insieme di partenza
   * @return il nuovo insieme
   */
  public Insieme<T> clone();
  
}
